#include "testApp.h"

//--------------------------------------------------------------
void testApp::setup(){
    ofBackground(255,255,255);
	ofSetFrameRate(30);
	//ofSetVerticalSync(true);
	//ofSetLogLevel(OF_LOG_VERBOSE);

    glEnable(GL_DEPTH_TEST);
    glDepthFunc(GL_LEQUAL);
    glShadeModel(GL_SMOOTH);

    width = 640;
	height = 480;

    bool connected = tcpClient.setup("192.168.10.8", 16000);
	//tcpClient.sendRaw("z\n"); // PanTilt Reset

  	// Uncomment this to show movies with alpha channels
	// roverMovie.setPixelFormat(OF_PIXELS_RGBA);
	roverMovie.loadMovie("http://192.168.10.8:8080/?action=stream");
	roverMovie.play();

    colorImage.allocate(width, height);
	grayImage.allocate(width, height);
	grayThres.allocate(width, height);
	grayImageFace.allocate(width/2, height/2);

    // This uses the default camera calibration and marker file
	artk.setup(width, height);

	threshold = 85;
	artk.setThreshold(threshold);

	stopPlay = false;

	posLR = 0;
	posUD = 0;
	scanL = true;
	scanR = false;
	scanLR = true;
	foundM0 = false;
	cameraM0 = false;
	roverM0 = false;
	roverMID0 = false;
	roverMID1 = false;
	roverMID2 = false;
	roverMID3 = false;
	roverMID4 = false;
	frameLoss = 0;

    // GUI
    moveAround.addListener(this,&testApp::moveAroundToggled);
    faceDetect.addListener(this,&testApp::faceDetectToggled);
    findMarker.addListener(this,&testApp::findMarkerToggled);
    thres.addListener(this,&testApp::thresChanged);
    pan.addListener(this,&testApp::panChanged);
    tilt.addListener(this,&testApp::tiltChanged);

    gui.setup("ARTURo", "ARTURo.xml", 810, 10); // most of the time you don't need a name
	gui.add(moveAround.setup("moveAround", true));
	gui.add(faceDetect.setup("faceDetect", false));
	gui.add(findMarker.setup("findMarker", false));
	gui.add(thres.setup("threshold", 85, 10, 99));
	gui.add(drawThres.setup("drawThreshold", false));
	gui.add(pan.setup("pan", 0, -85, 85));
	gui.add(tilt.setup("tilt", 0, -20, 85));

	bHide = true;

    // tcpClient was here

	finder.setup("haarcascade_frontalface_alt.xml");
	finder.setScaleHaar(1.4);
	finder.setNeighbors(2);

	img0.loadImage("logo-hacklab512.png");
	img1.loadImage("oshw-logo-m.png");
	img2.loadImage("baby_tux.png");
	img3.loadImage("arduino_logo.jpg");
	img4.loadImage("bg.png");
}

void testApp::moveAroundToggled(bool &moveA) {
    if (moveA) {
        faceDetect = false;
        findMarker = false;

        tcpClient.sendRaw("z\n");

        posLR = 0; pan = 0;
        posUD = 0; tilt = 0;
    }
}

void testApp::faceDetectToggled(bool &faceD) {
    if (faceD) {
        moveAround = false;
        findMarker = false;

        posLR = 0; pan = 0;
        posUD = 0; tilt = 0;

        tcpClient.sendRaw("z\n");
        for (int i = 0; i < 45; i++) {
            posUD++; tilt = tilt + 1;
            tcpClient.sendRaw("u\n");
        }
    }
}

void testApp::findMarkerToggled(bool &findM) {
    if (findM) {
        moveAround = false;
        faceDetect = false;

        tcpClient.sendRaw("z\n");

        posLR = 0; pan = 0;
        posUD = 0; tilt = 0;
        scanLR = true;
        foundM0 = false;
        cameraM0 = false;
        roverM0 = false;
        roverMID0 = false;
        roverMID1 = false;
        roverMID2 = false;
        roverMID3 = false;
        roverMID4 = false;
        frameLoss = 0;
    }
}

void testApp::thresChanged(int &thr) {
    threshold = thr;
    artk.setThreshold(threshold);
}

void testApp::panChanged(int &pan) {
    int pp = pan - posLR;

    if (pp > 0) {
        for (int i = 0; i < pp; i++) {
            posLR++;
            tcpClient.sendRaw("r\n");
        }
    } else if (pp < 0) {
        for (int i = pp; i < 0; i++) {
            posLR--;
            tcpClient.sendRaw("l\n");
        }
    }
}

void testApp::tiltChanged(int &tilt) {
    int pp = tilt - posUD;

    if (pp > 0) {
        for (int i = 0; i < pp; i++) {
            posUD++;
            tcpClient.sendRaw("u\n");
        }
    } else if (pp < 0) {
        for (int i = pp; i < 0; i++) {
            posUD--;
            tcpClient.sendRaw("d\n");
        }
    }
}

//--------------------------------------------------------------
void testApp::update(){

    roverMovie.update();

    if (roverMovie.isFrameNew()) {
        colorImage.setFromPixels(roverMovie.getPixels(), width, height);

        // convert our camera image to grayscale
		grayImage = colorImage;
		// apply a threshold so we can see what is going on
		grayThres = grayImage;
		grayThres.threshold(threshold);

		// Pass in the new image pixels to artk
		artk.update(grayImage.getPixels());

        if (faceDetect) {
            //grayImage.scale(0.5, 0.5);
            grayImage.resize(width/2, height/2);
            grayImageFace = grayImage;
            finder.findHaarObjects(grayImageFace, 20, 20, 280, 200, 32, 32);
            //cout << grayImage.width << " x " << grayImage.height << endl;
            grayImage.resize(width, height);
        }
    }

    if (moveAround) {

    } else if (faceDetect) {

    } else if (findMarker) {
        if (!roverMID0 && !roverMID1 && !roverMID2 && !roverMID3 && !roverMID4) {
            markerID = 0;
            roverMarker(markerID);
        }
        if (roverMID0 && !roverMID1 && !roverMID2 && !roverMID3 && !roverMID4) {
            markerID = 1;
            roverMarker(markerID);
        }
        if (roverMID0 && roverMID1 && !roverMID2 && !roverMID3 && !roverMID4) {
            markerID = 2;
            roverMarker(markerID);
        }
        if (roverMID0 && roverMID1 && roverMID2 && !roverMID3 && !roverMID4) {
            markerID = 3;
            roverMarker(markerID);
        }
        if (roverMID0 && roverMID1 && roverMID2 && roverMID3 && !roverMID4) {
            markerID = 4;
            roverMarker(markerID);
        }
    }
}

//--------------------------------------------------------------
void testApp::draw(){
    //ofEnableDepthTest();

	ofSetHexColor(0xFFFFFF);

    //roverMovie.draw(0, 0);
    colorImage.draw(0, 0);
    //grayImage.draw(0, 0);

//    if (faceDetect) {
//        grayImageFace.draw(0, 0);
//    } else {
//        colorImage.draw(0, 0);
//    }

    if (drawThres) {
        grayThres.draw(672, 216, 320, 240);
    }

    ofSetHexColor(0x000000);

    ofDrawBitmapString("PanTilt up:    'w'" , 650, 20);
    ofDrawBitmapString("PanTilt down:  's'" , 650, 40);
    ofDrawBitmapString("PanTilt left:  'a'" , 650, 60);
    ofDrawBitmapString("PanTilt right: 'd'" , 650, 80);
    ofDrawBitmapString("Forward:  'KEY_UP'" , 650, 120);
    ofDrawBitmapString("Backward: 'KEY_DN'" , 650, 140);
    ofDrawBitmapString("Left:     'KEY_LF" , 650, 160);
    ofDrawBitmapString("Right:    'KEY_RT'" , 650, 180);

    if (findMarker) {
        int ind0 = artk.getMarkerIndex(markerID);
        int id0 = artk.getMarkerID(0);
        ofVec3f camPos = artk.getCameraPosition(ind0);
        ofVec3f camTra = artk.getTranslation(ind0);
        ofQuaternion camQua = artk.getOrientationQuaternion(ind0);
        float angle, x, y, z;
        camQua.getRotate(angle, x, y, z);

        ofVec3f center = artk.getDetectedMarkerCenter(ind0);

        ofDrawBitmapString("pan-tilt: " + ofToString(posLR) + ", " + ofToString(posUD), 680, 280);
        ofDrawBitmapString("campos: " + ofToString(camPos.x, 2) + ", " + ofToString(camPos.y, 2) + ", " + ofToString(camPos.z, 2), 680, 300);
        ofDrawBitmapString("camtra: " + ofToString(camTra.x, 2) + ", " + ofToString(camTra.y, 2) + ", " + ofToString(camTra.z, 2), 680, 320);
        ofDrawBitmapString("camrot: " + ofToString(angle, 2) + ", " + ofToString(x, 2) + ", " + ofToString(y, 2) + ", " + ofToString(z, 2), 680, 340);
        ofDrawBitmapString("index: " + ofToString(ind0) + " id: " + ofToString(id0), 680, 360);
        ofDrawBitmapString("center: " + ofToString(center.x, 2) + ", " + ofToString(center.y, 2) + ", " + ofToString(center.z, 2), 680, 380);
        ofDrawBitmapString("diagonal: " + ofToString(diag), 680, 400);
    }

	if (bHide) {
		gui.draw();
	}

    if (faceDetect) {
        if (finder.blobs.size() > 0) {
            ofRectangle cur = finder.blobs[0].boundingRect;
            ofNoFill();
            ofSetLineWidth(4);
            ofSetColor(240, 30, 30, 200);
            ofRect(2 * cur.x, 2 * cur.y, 2 * cur.width, 2 * cur.height);

            int fcx = 2 * (cur.x + cur.width / 2);
            int fcy = 2 * (cur.y + cur.height / 2);

            if ((fcx - width / 2) > 50) {
                //pan.removeListener(this,&testApp::panChanged);
                posLR++;
                //pan = pan + 1;
                tcpClient.sendRaw("r\n");
                //pan.addListener(this,&testApp::panChanged);
            } else if ((fcx - width / 2) < -50) {
                //pan.removeListener(this,&testApp::panChanged);
                posLR--;
                //pan = pan - 1;
                tcpClient.sendRaw("l\n");
                //pan.addListener(this,&testApp::panChanged);
            }
            if ((fcy - height / 2) > 50) {
                //tilt.removeListener(this,&testApp::tiltChanged);
                posUD--;
                //tilt = tilt - 1;
                tcpClient.sendRaw("d\n");
                //tilt.addListener(this,&testApp::tiltChanged);
            } else if ((fcy - height / 2) < -50) {
                //tilt.removeListener(this,&testApp::tiltChanged);
                posUD++;
                //tilt = tilt + 1;
                tcpClient.sendRaw("u\n");
                //tilt.addListener(this,&testApp::tiltChanged);
            }
        }
    } else if (findMarker || moveAround) {
        // ARTK draw
        // An easy was to see what is going on
        // Draws the marker location and id number
        //artk.draw(0, 0);

        int myID0 = artk.getMarkerIndex(0);
        int myID1 = artk.getMarkerIndex(1);
        int myID2 = artk.getMarkerIndex(2);
        int myID3 = artk.getMarkerIndex(3);
        int myID4 = artk.getMarkerIndex(4);

        // ARTK 3D stuff
        // This is another way of drawing objects aligned with the marker
        // First apply the projection matrix once
        //ofPushMatrix();
        artk.applyProjectionMatrix(640, 480);
        // Find out how many markers have been detected
        int numDetected = artk.getNumDetectedMarkers();
        ofEnableAlphaBlending();
        // Draw for each marker discovered
        for(int i=0; i<numDetected; i++) {

            // Set the matrix to the perspective of this marker
            // The origin is in the middle of the marker
            artk.applyModelMatrix(i);

            // Draw a line from the center out
            ofEnableSmoothing();
            ofSetLineWidth(2);
            glEnable(GL_DEPTH_TEST);
            ofEnableNormalizedTexCoords();
            ofPushMatrix();
                ofTranslate(0, 0, 30);
                ofRotateX(ofGetFrameNum());
                ofRotateY(ofGetFrameNum());
                ofRotateZ(ofGetFrameNum());
                ofFill();
                ofSetColor(250, 250, 250, 220);
                if (i == myID0) {
                    img0.bind();
                } else if (i == myID1) {
                    img1.bind();
                } else if (i == myID2) {
                    img2.bind();
                } else if (i == myID3) {
                    img3.bind();
                } else if (i == myID4) {
                    img4.bind();
                }

                ofBox(20);

                if (i == myID0) {
                    img0.unbind();
                } else if (i == myID1) {
                    img1.unbind();
                } else if (i == myID2) {
                    img2.unbind();
                } else if (i == myID3) {
                    img3.unbind();
                } else if (i == myID4) {
                    img4.unbind();
                }
                ofNoFill();
                ofSetColor(30, 30, 30, 220);
                ofBox(20);
        //		glBegin(GL_LINES);
        //		glVertex3f(0, 0, 0);
        //		glVertex3f(0, 0, 50);
        //		glEnd();
            ofPopMatrix();

            // Draw a stack of rectangles by offseting on the z axis
            ofPushMatrix();
                for(int i=0; i<10; i++) {
                    ofNoFill();
                    ofSetColor(240, 60, 30, 150);
                    ofRect(-20, -20, 40, 40);
                    ofTranslate(0, 0, i * 1);
                }
            ofPopMatrix();
            ofDisableNormalizedTexCoords();
            glDisable(GL_DEPTH_TEST);
        }
        //ofPopMatrix();
    }
}

//--------------------------------------------------------------
void testApp::roverMarker(int id) {

    int ind0;

    if (scanLR && !(ofGetFrameNum() % 2)) {

        ind0 = artk.getMarkerIndex(id);

        if (ind0 > -1) {
            scanLR = false;
            foundM0 = true;
            cout << "found marker " << id << endl;
        }
        if (scanL && posLR > -85) {
            posLR--; pan = pan - 1;
            tcpClient.sendRaw("l\n");   // PanTilt sx
            if (posLR == -84) {
                scanL = false;
                scanR = true;
            }
        }
        if (scanR && posLR < 85) {
            posLR++; pan = pan + 1;
            tcpClient.sendRaw("r\n");   // PanTilt dx
            if (posLR == 84) {
                scanL = true;
                scanR = false;
            }
        }
    }

    if (foundM0 && !(ofGetFrameNum() % 2)) {

        ind0 = artk.getMarkerIndex(id);

        if (ind0 > -1) {
            center = artk.getDetectedMarkerCenter(ind0);
            if (center.x < (width / 2) - 20) {
                posLR--; pan = pan - 1;
                tcpClient.sendRaw("l\n");   // PanTilt dx
            } else if (center.x > (width / 2) + 20) {
                posLR++; pan = pan + 1;
                tcpClient.sendRaw("r\n");   // PanTilt sx
            } else {
                foundM0 = false;
                cameraM0 = true;
                cout << "camera direction OK" << endl;
            }
        }
    }

    if (cameraM0 && !(ofGetFrameNum() % 2)) {

        ind0 = artk.getMarkerIndex(id);

        if (ind0 > -1) {
            center = artk.getDetectedMarkerCenter(ind0);
            if (center.x < (width / 2) - 20) {
                posLR--; pan = pan - 1;
                tcpClient.sendRaw("l\n");   // PanTilt sx
            } else if (center.x > (width / 2) + 20) {
                posLR++; pan = pan + 1;
                tcpClient.sendRaw("r\n");   // PanTilt dx
            } else if (posLR == 0) {
                cameraM0 = false;
                roverM0 = true;
                frameLoss = 0;
                cout << "rover direction OK" << endl;
            }

            if (posLR < 0) {
                posLR++; pan = pan + 1;
                tcpClient.sendRaw("r\n");   // PanTilt dx
                tcpClient.sendRaw("s\n");   // Rover sx
            } else if (posLR > 0) {
                posLR--; pan = pan - 1;
                tcpClient.sendRaw("l\n");   // PanTilt sx
                tcpClient.sendRaw("e\n");   // Rover dx
            }
        } else {
            frameLoss++;
            if (frameLoss == 10) {
                scanLR = true;
                foundM0 = false;
                cameraM0 = false;
                roverM0 = false;
                frameLoss = 0;
            }
        }
    }

    if (roverM0 && !(ofGetFrameNum() % 2)) {

        ind0 = artk.getMarkerIndex(id);

        if (ind0 > -1) {
            center = artk.getDetectedMarkerCenter(ind0);
            artk.getDetectedMarkerOrderedBorderCorners(ind0, corners);

            diag = (corners[0].x - corners[2].x) * (corners[0].x - corners[2].x);
            diag += (corners[0].y - corners[2].y) * (corners[0].y - corners[2].y);
            diag += (corners[1].x - corners[3].x) * (corners[1].x - corners[3].x);
            diag += (corners[1].y - corners[3].y) * (corners[1].y - corners[3].y);
            diag /= 2.0;
            diag = sqrt(diag);

            if (center.x < (width / 2) - 60) {
                posLR--; pan = pan - 1;
                tcpClient.sendRaw("l\n");   // PanTilt sx
            } else if (center.x > (width / 2) + 60) {
                posLR++; pan = pan + 1;
                tcpClient.sendRaw("r\n");   // PanTilt dx
            } else if (posLR == 0) {
                if (diag < 350.0) {
                    tcpClient.sendRaw("a\n");   // Rover avanti
                    tcpClient.sendRaw("a\n");   // Rover avanti
                } else {
                    scanLR = true;
                    foundM0 = false;
                    cameraM0 = false;
                    roverM0 = false;
                    frameLoss = 0;
                    cout << "marker " << id << " OK" << endl;
                    if (id == 0) {
                        roverMID0 = true;
                    } else if (id == 1) {
                        roverMID1 = true;
                    } else if (id == 2) {
                        roverMID2 = true;
                    } else if (id == 3) {
                        roverMID3 = true;
                    } else if (id == 4) {
                        roverMID4 = true;
                    }
                }
            }

            if (posLR < 0) {
                posLR++; pan = pan + 1;
                tcpClient.sendRaw("r\n");   // PanTilt dx
                tcpClient.sendRaw("s\n");   // Rover sx
            } else if (posLR > 0) {
                posLR--; pan = pan - 1;
                tcpClient.sendRaw("l\n");   // PanTilt sx
                tcpClient.sendRaw("e\n");   // Rover dx
            }
        } else {
            frameLoss++;
            if (frameLoss == 10) {
                scanLR = true;
                foundM0 = false;
                cameraM0 = false;
                roverM0 = false;
                frameLoss = 0;
            }
        }
    }
}

//--------------------------------------------------------------
void testApp::keyPressed  (int key){
    switch(key){
        case 'p':
            stopPlay=!stopPlay;
            if (stopPlay) {
                roverMovie.stop();
            } else {
                roverMovie.play();
            }
        break;
        case 'z':
            tcpClient.sendRaw("z\n");
            posLR = 0; pan = 0;
            posUD = 0; tilt = 0;
            scanLR = true;
            foundM0 = false;
            cameraM0 = false;
            roverM0 = false;
            roverMID0 = false;
            roverMID1 = false;
        break;
        case OF_KEY_UP:
            if (moveAround) {
                tcpClient.sendRaw("a\n");   // Rover avanti
            }
        break;
        case OF_KEY_DOWN:
            if (moveAround) {
                tcpClient.sendRaw("i\n");   // Rover indietro
            }
        break;
        case OF_KEY_LEFT:
            if (moveAround) {
                tcpClient.sendRaw("s\n");   // Rover sx
            }
        break;
        case OF_KEY_RIGHT:
            if (moveAround) {
                tcpClient.sendRaw("e\n");   // Rover dx
            }
        break;
        case 'w':
            if (moveAround && posUD < 85) {
                posUD++; tilt = tilt + 1;
                tcpClient.sendRaw("u\n");   // PanTilt up
            }
        break;
        case 's':
            if (moveAround && posUD > -20) {
                posUD--; tilt = tilt - 1;
                tcpClient.sendRaw("d\n");   // PanTilt down
            }
        break;
        case 'a':
            if (moveAround && posLR > -85) {
                posLR--; pan = pan - 1;
                tcpClient.sendRaw("l\n");   // PanTilt sx
            }
        break;
        case 'd':
            if (moveAround && posLR < 85) {
                posLR++; pan = pan + 1;
                tcpClient.sendRaw("r\n");   // PanTilt dx
            }
        break;
        case 'm':
            threshold++;
            if (threshold > 99) {
                threshold = 99;
            }
            artk.setThreshold(threshold);
        break;
        case 'n':
            threshold--;
            if (threshold < 10) {
                threshold = 10;
            }
            artk.setThreshold(threshold);
        break;
        case 'h':
            bHide = !bHide;
        break;
    }
}

//--------------------------------------------------------------
void testApp::keyReleased(int key){

}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){

}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){

}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){

}


//--------------------------------------------------------------
void testApp::mouseReleased(int x, int y, int button){

}

//--------------------------------------------------------------
void testApp::windowResized(int w, int h){

}

//--------------------------------------------------------------
void testApp::gotMessage(ofMessage msg){

}

//--------------------------------------------------------------
void testApp::dragEvent(ofDragInfo dragInfo){

}
